<?php

namespace Drupal\migrate_faker\Plugin\migrate\process;

use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\migrate\MigrateExecutableInterface;
use Drupal\migrate\ProcessPluginBase;
use Drupal\migrate\Row;
use Faker\Factory;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Returns a value generated by the Faker library.
 *
 * Available configuration keys:
 * - generator: A method name to call in the faker generator.
 * - args: (optional) Arguments to pass to the generator.
 * - faker_locale: (optional) The language code to use with faker.
 * - faker_factory_service: (optional) A service to load a faker instance.
 *
 * Example:
 *
 * @code
 * process:
 *   body/value:
 *     -
 *       plugin: faker
 *       generator: paragraphs
 *       args:
 *        - 1
 *        - TRUE
 * @endcode
 *
 * @MigrateProcessPlugin(
 *   id = "faker",
 *   handle_multiples = TRUE
 * )
 */
final class Faker extends ProcessPluginBase implements ContainerFactoryPluginInterface {

  /**
   * The faker instance.
   *
   * @var mixed
   */
  protected $fakerService;

  /**
   * The locale to use to create faker content.
   *
   * @var string
   */
  protected $fakerLocale;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = new static($configuration, $plugin_id, $plugin_definition);
    $instance->fakerLocale = $configuration['faker_locale'] ?? '_faker_locale';
    if (!empty($configuration['faker_factory_service'])) {
      $instance->fakerService = $container->get($configuration['faker_factory_service']);
    }
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function transform($value, MigrateExecutableInterface $migrate_executable, Row $row, $destination_property) {
    $locale = $row->getSource()[$this->fakerLocale] ?? Factory::DEFAULT_LOCALE;
    if (empty($this->fakerService)) {
      $faker = Factory::create($locale);
    }
    else {
      $faker = $this->fakerService->create($locale);
    }
    $method_name = $this->configuration['generator'] ?? 'paragraphs';

    $args = $this->configuration['args'] ?? [];
    return call_user_func_array([$faker, $method_name], $args);
  }

}
